indexing

   description: "Base class for all Standard C or POSIX classes."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $";
   revision: "$Revision: 1.3 $"

class 

   STDC_BASE

inherit
   
   EXCEPTIONS
   
   STDC_CONSTANTS

   EXTERNAL_HELPER


feature -- errno
   
   errno: STDC_ERRNO is
      once
         create Result
      end
   

feature -- exceptions

   raise_posix_error is
      require
         -- errno_should_be_set: errno.is_not_ok
      do
         raise (errno.message)
      end

   safe_call (res: INTEGER) is
         -- raise an exception when value = -1
      do
         if res = -1 then
            raise_posix_error
         end
      end


feature {NONE} -- test or set bits in an integer
   
   flip_bits (bit_field, bits: INTEGER; on: BOOLEAN): INTEGER is
         -- change zero or more bits and return the result
      do
         if on then
            Result := posix_or (bit_field, bits)
         else
            Result := posix_and (bit_field, posix_not (bits))
         end
      ensure
         bits_flipped: (not on) or test_bits (Result, bits)
      end

   test_bits (bit_field, bits: INTEGER): BOOLEAN is
         -- True if given bits are 1 in bit_field
      do
         Result := posix_and (bit_field, bits) = bits
      end


feature {NONE} -- Make bit operations on integers possible
   
   posix_and (op1, op2: INTEGER): INTEGER is
      external "C"
      end

   posix_not (op: INTEGER): INTEGER is
      external "C"
      end

   posix_or (op1, op2: INTEGER): INTEGER is
      external "C"
      end


end -- class STDC_BASE
